// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-household/errors"
	dbmodel "property-household/model"
	pb_v1 "property-household/pb/v1"
)

func checkHouseholdGardenNameChangeParam(req *pb_v1.HouseholdGardenNameChangeRequest) error {
	switch {
	case req.GardenId < 1:
		return status.Error(10003, "小区id不能为空")
	case req.GardenName == "":
		return status.Error(10003, "小区名称不能为空")
	}
	return nil
}

// 废弃不用
func HouseholdGardenNameChange(ctx context.Context, req *pb_v1.HouseholdGardenNameChangeRequest) (reply *pb_v1.HouseholdGardenNameChangeReply, err error) {

	reply = &pb_v1.HouseholdGardenNameChangeReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkHouseholdGardenNameChangeParam(req)
	if err != nil {
		return nil, err
	}
	p := dbmodel.THouse{}
	where := map[string]interface{}{
		"garden_id": req.GardenId,
	}
	values := map[string]interface{}{
		"garden_name":req.GardenName,
	}
	err = p.Update(database.DB(), where, values)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil

}
