// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house_rent

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"property-household/errors"
	dbmodel "property-household/model"
	pb_v1 "property-household/pb/v1"
)

//
func HouseRentAppointmentDel(ctx context.Context, req *pb_v1.HouseRentAppointmentDelRequest) (reply *pb_v1.HouseRentAppointmentDelReply, err error) {

	reply = &pb_v1.HouseRentAppointmentDelReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	// 参数检查
	if req.Id < 1 && len(req.RentIds) == 0 {
		return nil, errors.ParamsError
	}

	p := dbmodel.THouseRentAppointment{}
	where := map[string]interface{}{}
	if req.Id > 0 {
		where["id"] = req.Id
	}
	if len(req.RentIds) > 0 {
		where["rent_id in"] = req.RentIds
	}

	err = p.Delete(database.DB(), where)
	if err != nil {
		return nil, errors.DataBaseError
	}

	return reply, nil
}
