// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package house_rent

import (
	"context"
	"encoding/json"
	"fmt"
	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
	"gorm.io/gorm"
	"property-household/errors"
	dbmodel "property-household/model"
	pb_v1 "property-household/pb/v1"
)

func checkHouseRentDownParam(req *pb_v1.HouseRentDownRequest) error {
	switch {
	case req.Id < 1:
		return status.Error(10003, "id不能为空")
	case req.GardenId < 1:
		return status.Error(10003, "小区不能为空")

	}

	return nil
}

//
func HouseRentDown(ctx context.Context, req *pb_v1.HouseRentDownRequest) (reply *pb_v1.HouseRentDownReply, err error) {

	reply = &pb_v1.HouseRentDownReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()

	err = checkHouseRentDownParam(req)
	if err != nil {
		return nil, err
	}
	p := dbmodel.THouseRent{}
	where := map[string]interface{}{
		"id": req.Id,
	}
	err = p.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if p.ID == 0 {
		return nil, errors.ErrRecordNotFound
	}
	if p.ApproveStatus != 2 {
		return nil, status.Error(10003, "当前状态不能下架")
	}

	values := map[string]interface{}{
		"approve_status": 4,
	}

	db := database.DB().Begin()
	err = p.Update(db, where, values)
	if err != nil {
		db.Rollback()
		return nil, errors.DataBaseError
	}

	p.ApproveStatus = 4
	err = houseRentSync(p, false, req.GardenId, -1)
	if err != nil {
		db.Rollback()
		return nil, err
	}
	db.Commit()

	return reply, nil
}
