// Copyright 2019 getensh.com. All rights reserved.
// Use of this source code is governed by getensh.com.

package user

import (
	"context"
	"encoding/json"
	"fmt"
	"gorm.io/gorm"
	"property-household/errors"
	dbmodel "property-household/model"
	pb_v1 "property-household/pb/v1"
	"property-household/utils"

	"git.getensh.com/common/gopkgs/database"
	"git.getensh.com/common/gopkgs/logger"
	"go.uber.org/zap"
	"google.golang.org/grpc/status"
)

const (
	HouseholdApproveStatusWait    = 1
	HouseholdApproveStatusSuccess = 2
	HouseholdApproveStatusFail    = 3
)

// 实名认证
func RealNameAuth(ctx context.Context, req *pb_v1.RealNameAuthRequest) (reply *pb_v1.RealNameAuthReply, err error) {
	reply = &pb_v1.RealNameAuthReply{}

	// 捕获各个task中的异常并返回给调用者
	defer func() {
		if r := recover(); r != nil {
			err = fmt.Errorf("%+v", r)
			e := &status.Status{}
			if er := json.Unmarshal([]byte(err.Error()), e); er != nil {
				logger.Error("err",
					zap.String("system_err", err.Error()),
					zap.Stack("stacktrace"))
			}
		}
	}()
	if req.IdNumber == "" || req.IdType < 1 || req.Uid < 1 || req.RealName == "" {
		return nil, errors.ParamsError
	}
	if req.IdType == 1 && !utils.CheckIDCert(req.IdNumber) {
		return nil, status.Error(10003, "身份证格式错误")
	}
	user := &dbmodel.TUser{}
	where := map[string]interface{}{
		"id": req.Uid,
	}
	err = user.Find(database.DB(), where)
	if err != nil && err != gorm.ErrRecordNotFound {
		return nil, errors.DataBaseError
	}
	if user.ID == 0 {
		return nil, errors.UserNotExist
	}
	idNumber := ""
	if req.IdNumber != "" {
		idNumber = utils.CertEncrypt(req.IdNumber)
	}
	uinfo := dbmodel.TUser{IdType: req.IdType, IdNumber: idNumber, RealName: req.RealName}
	if err = userChange(req.Uid, uinfo); err != nil {
		return nil, err
	}

	return reply, nil
}
